#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import numpy as np

from redcp import * # import data types

DELTA_DISTANCE = 0.12
C_TURNING_SPEED = 5
C_AVOID_SPEED = 10

class Controller:
    def __init__(self):
        pass

    def goto(self, goal: Pose, odometry: Pose, collision: bool) -> Twist:
        """Method to steer the robot towards the goal position given its current 
           odometry and collision status
        Args:
            goal: Pose of the robot goal
            odometry: Perceived odometry of the robot
            collision: bool of the robot collision status
        Returns:
            cmd: Twist steering command
        """
        # zero velocity steering command
        if odometry is None or goal is None: return Twist()
        if collision: return None

        cmd_msg = Twist()
        
        ## get position diff for distance calculation
        dx = goal.position.x - odometry.pose.position.x
        dy = goal.position.y - odometry.pose.position.y
        
        ## phi from robot to goal
        gphi = math.atan2(dy, dx)

        ## current robot orientation
        rphi = odometry.pose.orientation.to_Euler()[0]

        ## phi difference eddited with Milos conversion
        dphi = ((gphi-rphi + math.pi) % (2*math.pi)) - math.pi 

        dist = odometry.pose.dist(goal)
        
        ## set message values for robot to move
        cmd_msg.linear.x = dist
        cmd_msg.angular.z = dphi * C_TURNING_SPEED

        ## difference between robot and goal orientation
        ## UNUSED
        ghphi = goal.orientation.to_Euler()[0]
        ghphi = ((ghphi-rphi + math.pi) % (2*math.pi)) - math.pi 

        if dist < DELTA_DISTANCE:
            return None

        return cmd_msg            

    def goto_reactive(self, goal: Pose, odometry: Odometry, collisions: bool, laser_scan: LaserScan) -> Twist:
        """Method to steer the robot towards the goal position while avoiding 
           contact with the obstacles given its current odometry, collision 
           status and laser scan data
        Args:
            goal: Pose of the robot goal
            odometry: Perceived odometry of the robot
            collision: bool of the robot collision status
            laser_scan: LaserScan data perceived by the robot
        Returns:
            cmd: Twist steering command
        """
        #zero velocity steering command
        cmd_msg = Twist()

        
        return cmd_msg
