###################################################
#Helper functions
###################################################

import numpy as np

from bresenham import bresenham_line

def check_path_validity(path, gridmap, EPS):
    obstacle_map = gridmap.data.reshape(gridmap.height, gridmap.width)
    valid = True
    for i in range(len(path.poses)-1):
        p1 = gridmap.world_to_map(path.poses[i])
        p2 = gridmap.world_to_map(path.poses[i+1])

        line = bresenham_line(p1,p2)
        for pose in line:
            (x,y) = pose
            if x < 0 or x >= gridmap.width or y < 0 and y >= gridmap.height:
                valid = False
                break
            else:
                if obstacle_map[y,x] > 0.5 + EPS:
                    valid = False
                    break
        if not valid:
            break
    return valid    

def plot_map(plt, gridmap, title):
    fig = plt.figure()
    ax=plt.gca()
    gridmap.plot(ax)
    plt.xlabel('x[m]')
    plt.ylabel('y[m]')
    plt.axis('square')
    plt.title(title)
    return fig

def save_image(fig, msg, image_file):
    fig.savefig(image_file, bbox_inches='tight')
    print(msg, image_file) 
    return

def get_path_length(path):
    length= 0
    if path is not None:
        poses = np.asarray([(pose.position.x,pose.position.y) for pose in path.poses])
        for i in range(1,len(poses)): #calculate the length
            length += np.linalg.norm(poses[i,:]-poses[i-1,:])
    return length

def draw_path(path, ax, clr):
    if path is not None and len(path.poses):
        poses = np.asarray([(pose.position.x,pose.position.y) for pose in path.poses])
        ax.plot(poses[:,0], poses[:,1], '.',color=clr)
        ax.plot(poses[:,0], poses[:,1], '-',color=clr)
        ax.plot(poses[0,0], poses[0,1], 'o', color='g')
        ax.plot(poses[-1,0], poses[-1,1], 'o', color='r')
    return

def plot_paths(plt, path, path_simple, gridmap, title):
    fig = plt.figure()
    ax = plt.gca()
    gridmap.plot(ax)
    draw_path(path, ax,'r')
    if path_simple is not None:
        draw_path(path_simple,ax,'b')
    plt.xlabel('x[m]')
    plt.ylabel('y[m]')
    plt.axis('square')
    plt.title(title)
    return fig

