#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import matplotlib.pyplot as plt

import sys
import os
import math
import numpy as np

sys.path.append('../')
sys.path.append('../redcp/hexapod_robot')
 
from redcp import * # import data types

import HexapodRobot as hexapod

from ControllerReactive import Controller 
from Mapper import Mapper

from utils import *

if __name__=="__main__":

    robot = hexapod.HexapodRobot(Controller())
    mapper = Mapper()

    robot.turn_on() #turn on the robot 
    robot.start_navigation() #start navigation thread
    goals = [ #assign goal for navigation
        Pose(Vector3(3.5, 3.5, 0), Quaternion(1, 0, 0, 0)), 
        Pose(Vector3(0.0, 0.0, 0), Quaternion(1, 0, 0, 0)), 
    ]
    
    plt.ion() #prepare the online plot
    fig, ax = plt.subplots()

    gridmap = OccupancyGrid() #prepare the gridmap
    gridmap.resolution = 0.1

    gridmap.width = 100
    gridmap.height = 100
    gridmap.origin = Pose(Vector3(-5.0, -5.0, 0.0), Quaternion(1, 0, 0, 0))
    gridmap.data = 0.5*np.ones((gridmap.height*gridmap.width))

    for goal in goals: #go from goal to goal
        #TODO test robot.goto(goal) 
        robot.goto_reactive(goal)
        while robot.navigation_goal is not None:
            plt.cla()
            #get the current laser scan and odometry and fuse them to the map
            gridmap = mapper.fuse_laser_scan(gridmap, robot.laser_scan_, robot.odometry_)
            redraw_map(ax, gridmap) #plot the map
    robot.stop_navigation()
    robot.turn_off()
