#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import matplotlib
matplotlib.use('Agg')

import os
import sys
import numpy as np

sys.path.append('../')

from redcp import * # import data types

import pickle #pickle for reading .pkl

from utils import *

from Mapper import Mapper

# Results dir
RESULT_DIR = 'results'

if __name__=="__main__":
    if not os.path.exists(RESULT_DIR): 
        os.mkdir(RESULT_DIR)

    ###################################################
    mapper = Mapper()

    #prepare the gridmap
    gridmap = OccupancyGrid()
    gridmap.resolution = 0.1
    #simple variant with predefined grid
    gridmap.width = 100
    gridmap.height = 100
    gridmap.origin = Pose(Vector3(-5.0,-5.0,0.0), Quaternion(1,0,0,0))
    gridmap.data = 0.5*np.ones((gridmap.height*gridmap.width))

    #load the dataset
    dataset = pickle.load( open( "data/scan_dataset.pkl", "rb" ) )
    reference = pickle.load( open( "data/gridmap.pkl", "rb" ) )
    print('Loaded dataset and reference')

    scan_n = 0
    for scan, odometry in dataset: #process the dataset
        sys.stdout.write('\rFusing scan %3d' % scan_n)
        scan_n += 1
        gridmap = mapper.fuse_laser_scan(gridmap, scan, odometry) #get the current laser scan and odometry and fuse them to the map
    print('\nAll scan fused compared to the map with reference')
    fig = plot_map(gridmap, 'Map - Created map')
    image_file = RESULT_DIR + '/map-fused.png'
    save_image(fig, 'Save created map to ' + image_file, image_file)
    
    #reference map
    fig = plot_map(reference, 'Map - Reference map')
    image_file = RESULT_DIR + '/map-reference.png'
    save_image(fig, 'Save reference map to ' + image_file, image_file)
        
    #compare map
    if gridmap.width != reference.width or gridmap.height != reference.height:
        print("ERROR: Different size of the student solution map (%d,%d) and the reference map (%d,%d)" % (gridmap.width, gridmap.height, reference.width, reference.height))
        print("ERROR: Unable to compare - will be assessed manualy")
    else:
        data = reference.data.reshape(reference.height, reference.width)
        data_g = gridmap.data.reshape(gridmap.height, gridmap.width)
        data = np.abs(data-data_g)
        reference.data = data.flatten()

        diff = np.sum(data[:])
        fig = plot_map(reference, 'Map - Difference fused and reference maps')
        image_file = RESULT_DIR + '/map-difference.png'
        save_image(fig, 'Save difference of fused and reference map to ' + image_file, image_file)
        print("Map difference: %.2f" % diff)
        
        if abs(diff) > 2000:
            print("WARN: The map difference is really large. It seems that there is something wrong with your solution. If you think that the evaluation script is wrong and your solution is correct, contact the lab teachers to check your solution manually.")
        else:
            print("INFO: The map difference under the expected threshold.")
            print("Solution OK!")

