#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import math
import time
import copy
import numpy as np

from redcp import * # import data types

import scipy.ndimage as ndimg
import skimage.measure as skm

CELL_FREE = 0.4 ## < value of free cell
CELL_OCCU = 0.6 ## > value of occupied cell

class Frontiers:
    def __init__(self):
        pass

    def find_free_edge_frontiers_representatives(self, grid_map: OccupancyGrid):
        """Method to find the representatives of free-edge frontiers (edge clusters between the free and unknown areas)
        Args:
            grid_map: OccupancyGrid - gridmap of the environment
        Returns:
            pose_list: Pose[] - list of selected representatives of the free edges
        """
        if grid_map is None:
            return []
        grid_map = grid_map.clone() # Clone the grid_map to avoid changes to the input grid
        grid = grid_map.data.reshape((grid_map.height, grid_map.width))
        grid = np.where(grid <= CELL_FREE, 0, np.where(grid >= CELL_OCCU, 1, 10)) # set 10 for unknown have convolution above 8 with at least one unknown cell 
        mask = np.array([[1, 1, 1], [1, 0, 1], [1, 1, 1]]) # 8-neigh
        grid_conv = ndimg.convolve(grid, mask, mode = 'constant', cval = 0.0)
        
        # threshold free space cell (0) and cell incident with unknown 
        grid = np.where(np.logical_and(grid == 0, grid_conv > 8), 1, 0) # set 1 for frontier cells
        labeled_image, num_labels = skm.label(grid, connectivity = 2, return_num=True)
        centroids = []
        for label in range(1, num_labels + 1):
            region = np.argwhere(labeled_image == label)
            centroids.append(tuple(reversed(region.mean(axis = 0))))
        centroid_poses = [grid_map.map_to_world(c) for c in centroids]
        return centroid_poses

