#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import sys
import copy
import numpy as np

sys.path.append('../')

from redcp import * # import data types

import pickle #pickle for reading .pkl

from Frontiers import Frontiers
from utils import *

RESULT_DIR = 'results'

# Evaluate frontiers determination
if __name__=="__main__":
    if not os.path.exists(RESULT_DIR): os.mkdir(RESULT_DIR)
        
    frontiers = Frontiers()

    dataset = pickle.load( open( "data/frontier_detection_eval.pkl", "rb" ) )
    reference_frontiers = pickle.load( open( "data/frontier_reference.pkl", "rb" ) )

    for idx, gridmap in enumerate(dataset):
        print("Process scenario %2d" % idx)
        goal_locations_ref = reference_frontiers[idx] #load the reference result
        if goal_locations_ref is None:
            print("WARN: Reference goal locations None!")
            break 
        
        goal_locations = frontiers.find_free_edge_frontiers_representatives(gridmap) #find free edges
        if goal_locations is None:
            print("WARN: Goal locations None!")
            break 

        n_locations_ref = len(goal_locations_ref)
        n_locations = n_wrong_locations = len(goal_locations)
        for location in goal_locations:
            for location_ref in goal_locations_ref: #check if it is close to reference
                if np.linalg.norm((location_ref.position.x - location.position.x, location_ref.position.y - location.position.y)) < 0.1: 
                    n_wrong_locations -= 1
                    break
        if n_locations != n_locations_ref:
            print('ERROR: Different number of found frontiers ' + str(n_locations) + ' of ' + str(n_locations_ref) + ' reference goal locations!')
        elif n_wrong_locations > 0:
            print("ERROR: Some of the found frontiers have wrong position")
        else:
            print("Scenario %d frontiers detected fine!" % idx)
        print('')
        fig = draw_goal_locations(gridmap, goal_locations, goal_locations_ref, 'Frontiers Detection - Scenario ' + str(idx))
        image_file = RESULT_DIR + '/frontiers-' + str(idx) + '.png'
        save_image(fig, 'Save reference and detected goal frontiers locations to ' + image_file, image_file)

